import logging
import uuid
import json
import os

from logging.config import dictConfig
from pathlib import Path

from platform_agent.lib.ctime import now

logger = logging.getLogger()


class PublishLogToSessionHandler(logging.Handler):
    def __init__(self, session):
        logging.Handler.__init__(self)
        self.session = session
        self.log_id = str(uuid.uuid4())

    def emit(self, record):
        if not self.session.active:
            return
        self.session.send_log(json.dumps({
            'id': self.log_id,
            'executed_at': now(),
            'type': 'LOGGER',
            'data': {'severity': record.levelname, 'message': record.getMessage()}
        }))


def configure_logger():

    log_path = "/var/log/noia-platform"

    log_file = Path(f"{log_path}/agent.log")

    noia_platform_dir = Path(f"{log_path}")

    if not noia_platform_dir.is_dir():
        noia_platform_dir.mkdir()
    if not log_file.is_file():
        log_file.write_text('')

    logging_config = dict(
        version=1,
        formatters={
            'f': {
                'format': '%(asctime)-24s %(levelname)-8s %(message)s'
            }
        },
        handlers={
            'h': {
                'class': 'logging.StreamHandler',
                'formatter': 'f',
                'level': int(os.environ.get('NOIA_LOG_LEVEL', 10))
            },
            'file': {
                'level': 'DEBUG',
                'class': 'logging.handlers.RotatingFileHandler',
                'formatter': 'f',
                'filename': os.environ.get('NOIA_LOG_FILE', "/var/log/noia-platform/agent.log")
            }
        },
        root={
            'handlers': ['h', 'file'],
            'level': int(os.environ.get('NOIA_LOG_LEVEL', 10)),
        },
    )

    dictConfig(logging_config)





def get_peer_info(ifname, wg):

    results = {}
    ss = wg.info(ifname)
    wg_info = dict(ss[0]['attrs'])
    peers = wg_info.get('WGDEVICE_A_PEERS', [])
    for peer in peers:
        peer = dict(peer['attrs'])
        results[peer['WGPEER_A_PUBLIC_KEY'].decode('utf-8')] = [allowed_ip['addr'] for allowed_ip in
                                                                peer['WGPEER_A_ALLOWEDIPS']]
    return results





import logging

from pyroute2 import IPRoute, NetlinkError

logger = logging.getLogger()


def ip_route_add(ifname, ip_list, gw_ipv4):
    ip_route = IPRoute()
    devices = ip_route.link_lookup(ifname=ifname)
    dev = devices[0]
    for ip in ip_list:
        try:
            ip_route.route('add', dst=ip, gateway=gw_ipv4, oif=dev)
        except NetlinkError as error:
            if error.code != 17:
                raise
            logger.info(f"[WG_CONF] add route failed [{ip}] - already exists")


def ip_route_del(ifname, ip_list, scope=None):
    ip_route = IPRoute()
    devices = ip_route.link_lookup(ifname=ifname)
    dev = devices[0]
    for ip in ip_list:
        try:
            ip_route.route('del', dst=ip, oif=dev, scope=scope)
        except NetlinkError as error:
            if error.code != 17:
                raise
            logger.info(f"[WG_CONF] del route failed [{ip}] - does not exist")


def create_rule(internal_ip, rt_table_id):
    ip_route = IPRoute()
    ip_route.flush_rules(table=rt_table_id)
    ip_route.flush_routes(table=rt_table_id)
    ip_route.rule('add', src=internal_ip, table=rt_table_id)
